﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;
using System.Net;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace NewAgeDesign.FileManager
{
    public class FileData
    {
        public string FileName;
        public string Extension;
        public string RelativePath;
        public string RelativeThumbPath;
        public int KB;

        public FileData()
        {
        }

        public FileData(string filePath)
        {
            FileName = Path.GetFileName(filePath);
            Extension = Path.GetExtension(filePath).TrimStart('.').ToLower();
            KB = Convert.ToInt32(new System.IO.FileInfo(filePath).Length / 1024);

            string path = "/" + Path.GetDirectoryName(filePath).Replace(HttpContext.Current.Server.MapPath("/"), "").Replace("\\", "/");
            RelativePath = Path.Combine(path, Path.GetFileName(filePath));
            RelativeThumbPath = Path.Combine(Path.Combine(path, "_thumbs"), Path.GetFileName(filePath));

            if (!File.Exists(HttpContext.Current.Server.MapPath(RelativeThumbPath)))
                RelativeThumbPath = "img/filetypes/" + Path.GetExtension(filePath).TrimStart('.') + ".png";

            if (!File.Exists(HttpContext.Current.Server.MapPath(RelativeThumbPath)))
                RelativeThumbPath = "img/filetypes/_blank.png";
        }
    }

    public partial class ExplorerEngine : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            string dir, file;
            switch (Request.Params["action"])
            {
                case "getfiles":
                    dir = "\\" + Server.UrlDecode(Request.Params["dir"]);
                    dir = Server.MapPath(dir);
                    List<FileData> files = new List<FileData>();
                    foreach (string f in Directory.GetFiles(dir))
                    {
                        GenerateThumb(f);
                        files.Add(new FileData(f));
                    }
                    Response.Write(ToJSON(files));
                    Response.End();
                    break;
                case "delete":
                    file = "\\" + Server.UrlDecode(Request.Params["file"]);
                    file = Server.MapPath(file);
                    File.Delete(file);

                    string tb = Path.Combine(Path.Combine(Path.GetDirectoryName(file), "_thumbs"), Path.GetFileName(file));
                    if (File.Exists(tb)) File.Delete(tb);
                    break;
                case "imageSize":
                    file = "\\" + Server.UrlDecode(Request.Params["file"]);
                    file = Server.MapPath(file);
                    System.Drawing.Image ImageOrig = System.Drawing.Image.FromFile(file);
                    string size = ImageOrig.Width + " x " + ImageOrig.Height;
                    Response.Write(ToJSON(size));
                    Response.End();
                    break;
                case "resizeImage":
                    file = "\\" + Server.UrlDecode(Request.Params["file"]);
                    file = Server.MapPath(file);
                    int x = Convert.ToInt32(Request.Params["x"]);
                    int y = Convert.ToInt32(Request.Params["y"]);
                    bool n = Convert.ToBoolean(Request.Params["n"]);

                    string newFile = file;
                    if (n)
                    {
                        while(File.Exists(newFile))
                        {
                            newFile = Path.Combine(Path.GetDirectoryName(newFile), "0" + Path.GetFileName(newFile));
                        }
                    }

                    ResizeImage(file, newFile, x, y);
                    Response.Write(ToJSON("OK"));
                    Response.End();
                    break;
            }
        }


        static public string ToJSON(object obj)
        {
            System.Web.Script.Serialization.JavaScriptSerializer serializer = new System.Web.Script.Serialization.JavaScriptSerializer();
            return serializer.Serialize(obj);
        }

        private void GenerateThumb(string filePath)
        {
            string[] extensiones = {
                                        ".jpg",
                                        ".png",
                                        ".bmp",
                                        ".gif"
                                    };

            if (!extensiones.Contains(Path.GetExtension(filePath).ToLower()))
                return;

            string thumbsDirectory = Path.Combine(Path.GetDirectoryName(filePath), "_thumbs");
            if (!Directory.Exists(thumbsDirectory)) Directory.CreateDirectory(thumbsDirectory);

            string thumbFile = Path.Combine(thumbsDirectory, Path.GetFileName(filePath));
            if (File.Exists(thumbFile)) return;

            ResizeImage(filePath, thumbFile, 100, 75);
        }


        public static string ResizeImage(string strImgPath, string strImgOutputPath, int iWidth, int iHeight)
        {
            try
            {
                bool mismaImagen = strImgPath.Equals(strImgOutputPath);
                if (mismaImagen)
                {
                    strImgOutputPath = strImgPath + "___.jpg";
                }

                string[] extensiones = {
                                           ".jpg",
                                           ".png",
                                           ".bmp",
                                           ".gif"
                                       };

                if (!extensiones.Contains(Path.GetExtension(strImgPath).ToLower()))
                    throw new Exception("Extensión no soportada");

                //Lee el fichero en un stream
                Stream mystream = null;

                if (strImgPath.StartsWith("http"))
                {
                    HttpWebRequest wreq = (HttpWebRequest)WebRequest.Create(strImgPath);
                    HttpWebResponse wresp = (HttpWebResponse)wreq.GetResponse();
                    mystream = wresp.GetResponseStream();
                }
                else
                    mystream = File.OpenRead(strImgPath);

                // Cargo la imágen
                Bitmap imgToResize = new Bitmap(mystream);

                Size size = new Size(iWidth, iHeight);

                int sourceWidth = imgToResize.Width;
                int sourceHeight = imgToResize.Height;

                float nPercent = 0;
                float nPercentW = 0;
                float nPercentH = 0;

                nPercentW = ((float)size.Width / (float)sourceWidth);
                nPercentH = ((float)size.Height / (float)sourceHeight);

                if (nPercentH < nPercentW)
                    nPercent = nPercentH;
                else
                    nPercent = nPercentW;

                int destWidth = (int)(sourceWidth * nPercent);
                int destHeight = (int)(sourceHeight * nPercent);

                if (destWidth <= imgToResize.Width || destHeight <= imgToResize.Height)
                {
                    Bitmap b = new Bitmap(destWidth, destHeight);
                    Graphics g = Graphics.FromImage((System.Drawing.Image)b);
                    g.InterpolationMode = InterpolationMode.HighQualityBicubic;

                    g.DrawImage(imgToResize, 0, 0, destWidth, destHeight);
                    g.Dispose();

                    // We will store the correct image codec in this object
                    ImageCodecInfo ici = GetEncoderInfo("image/jpeg"); ;
                    // This will specify the image quality to the encoder
                    EncoderParameter epQuality = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, Preferences.ImgCompression);
                    // Store the quality parameter in the list of encoder parameters
                    EncoderParameters eps = new EncoderParameters(1);
                    eps.Param[0] = epQuality;
                    b.Save(strImgOutputPath, ici, eps);
                    b.Dispose();
                    g.Dispose();
                }
                else if (strImgPath != strImgOutputPath)
                    File.Copy(strImgPath, strImgOutputPath);

                imgToResize.Dispose();
                mystream.Close();
                mystream.Dispose();
                imgToResize = null;
                mystream = null;
                GC.Collect();

                if (mismaImagen)
                {
                    System.Threading.Thread.Sleep(100); // Esperamos para que se libere con seguridad la imágen
                    File.Delete(strImgPath);
                    File.Move(strImgOutputPath, strImgPath);
                }

                return strImgPath;
            }
            catch(Exception ex)
            {
                // Dejo que siga porque sino se jode todo
                //throw;
                return String.Empty;
            }
        }

        private static ImageCodecInfo GetEncoderInfo(String mimeType)
        {
            int j;
            ImageCodecInfo[] encoders;
            encoders = ImageCodecInfo.GetImageEncoders();
            for (j = 0; j < encoders.Length; ++j)
            {
                if (encoders[j].MimeType == mimeType)
                    return encoders[j];
            }
            return null;
        }
    }
}